@@ -1,4 +1,23 @@
 /* -*- Mode: C; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
+/*
+ * libdatrie - Double-Array Trie Library
+ * Copyright (C) 2006  Theppitak Karoonboonyanan <thep@linux.thai.net>
+ *
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
 /*
  * alpha-map.c - map between character codes and trie alphabet
  * Created: 2006-08-19
@@ -15,6 +34,13 @@
 #include "alpha-map-private.h"
 #include "fileutils.h"
 
+/**
+ * @brief Alphabet string length
+ *
+ * @param str   : the array of null-terminated AlphaChar string to measure
+ *
+ * @return the total characters in @a str.
+ */
 int
 alpha_char_strlen (const AlphaChar *str)
 {
@@ -60,6 +86,16 @@ static int  alpha_map_get_total_ranges (const AlphaMap *alpha_map);
  * - INT32: range end
  */
 
+/**
+ * @brief Create new alphabet map
+ *
+ * @return a pointer to the newly created alphabet map, NULL on failure
+ *
+ * Create a new empty alphabet map. The map contents can then be added with
+ * alpha_map_add_range().
+ *
+ *  The created object must be freed with alpha_map_free().
+ */
 AlphaMap *
 alpha_map_new ()
 {
@@ -74,6 +110,15 @@ alpha_map_new ()
     return alpha_map;
 }
 
+/**
+ * @brief Create a clone of alphabet map
+ *
+ * @param a_map : the source alphabet map to clone
+ *
+ * @return a pointer to the alphabet map clone, NULL on failure
+ *
+ *  The created object must be freed with alpha_map_free().
+ */
 AlphaMap *
 alpha_map_clone (const AlphaMap *a_map)
 {
@@ -94,6 +139,13 @@ alpha_map_clone (const AlphaMap *a_map)
     return alpha_map;
 }
 
+/**
+ * @brief Free an alphabet map object
+ *
+ * @param alpha_map : the alphabet map object to free
+ *
+ * Destruct the @a alpha_map and free its allocated memory.
+ */
 void
 alpha_map_free (AlphaMap *alpha_map)
 {
@@ -119,28 +171,32 @@ alpha_map_read_bin (FILE *file)
 
     /* check signature */
     save_pos = ftell (file);
-    if (!file_read_int32 (file, (int32 *) &sig) || ALPHAMAP_SIGNATURE != sig) {
-        fseek (file, save_pos, SEEK_SET);
-        return NULL;
-    }
+    if (!file_read_int32 (file, (int32 *) &sig) || ALPHAMAP_SIGNATURE != sig)
+        goto exit_file_read;
 
-    alpha_map = alpha_map_new ();
-    if (!alpha_map)
-        return NULL;
+    if (NULL == (alpha_map = alpha_map_new ()))
+        goto exit_file_read;
 
     /* read number of ranges */
-    file_read_int32 (file, &total);
+    if (!file_read_int32 (file, &total))
+        goto exit_map_created;
 
     /* read character ranges */
     for (i = 0; i < total; i++) {
         int32   b, e;
 
-        file_read_int32 (file, &b);
-        file_read_int32 (file, &e);
+        if (!file_read_int32 (file, &b) || !file_read_int32 (file, &e))
+            goto exit_map_created;
         alpha_map_add_range (alpha_map, b, e);
     }
 
     return alpha_map;
+
+exit_map_created:
+    alpha_map_free (alpha_map);
+exit_file_read:
+    fseek (file, save_pos, SEEK_SET);
+    return NULL;
 }
 
 static int
@@ -178,6 +234,16 @@ alpha_map_write_bin (const AlphaMap *alpha_map, FILE *file)
     return 0;
 }
 
+/**
+ * @brief Add a range to alphabet map
+ *
+ * @param alpha_map : the alphabet map object
+ * @param begin     : the first character of the range
+ * @param end       : the last character of the range
+ *
+ * Add a range of character codes from @a begin to @a end to the
+ * alphabet set.
+ */
 int
 alpha_map_add_range (AlphaMap *alpha_map, AlphaChar begin, AlphaChar end)
 {
