@@ -1,4 +1,23 @@
 /* -*- Mode: C; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
+/*
+ * libdatrie - Double-Array Trie Library
+ * Copyright (C) 2006  Theppitak Karoonboonyanan <thep@linux.thai.net>
+ *
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
 /*
  * darray.c - Double-array trie structure
  * Created: 2006-08-13
@@ -7,6 +26,7 @@
 
 #include <string.h>
 #include <stdlib.h>
+#include <stdint.h>
 #include <stdio.h>
 
 #include "trie-private.h"
@@ -156,6 +176,11 @@ struct _DArray {
  */
 #define DA_POOL_BEGIN 3
 
+/**
+ * @brief Create a new double-array object
+ *
+ * Create a new empty doubla-array object.
+ */
 DArray *
 da_new ()
 {
@@ -183,6 +208,17 @@ exit_da_created:
     return NULL;
 }
 
+/**
+ * @brief Read double-array data from file
+ *
+ * @param file : the file to read
+ *
+ * @return a pointer to the openned double-array, NULL on failure
+ *
+ * Read double-array data from the opened file, starting from the current
+ * file pointer until the end of double array data block. On return, the
+ * file pointer is left at the position after the read block.
+ */
 DArray *
 da_read (FILE *file)
 {
@@ -192,34 +228,48 @@ da_read (FILE *file)
 
     /* check signature */
     save_pos = ftell (file);
-    if (!file_read_int32 (file, &n) || DA_SIGNATURE != (uint32) n) {
-        fseek (file, save_pos, SEEK_SET);
-        return NULL;
-    }
+    if (!file_read_int32 (file, &n) || DA_SIGNATURE != (uint32) n)
+        goto exit_file_read;
 
-    d = (DArray *) malloc (sizeof (DArray));
-    if (!d)
-        return NULL;
+    if (NULL == (d = (DArray *) malloc (sizeof (DArray))))
+        goto exit_file_read;
 
     /* read number of cells */
-    file_read_int32 (file, &d->num_cells);
-    d->cells     = (DACell *) malloc (d->num_cells * sizeof (DACell));
+    if (!file_read_int32 (file, &d->num_cells))
+        goto exit_da_created;
+    if (d->num_cells > SIZE_MAX / sizeof (DACell))
+        goto exit_da_created;
+    d->cells = (DACell *) malloc (d->num_cells * sizeof (DACell));
     if (!d->cells)
         goto exit_da_created;
     d->cells[0].base = DA_SIGNATURE;
     d->cells[0].check= d->num_cells;
     for (n = 1; n < d->num_cells; n++) {
-        file_read_int32 (file, &d->cells[n].base);
-        file_read_int32 (file, &d->cells[n].check);
+        if (!file_read_int32 (file, &d->cells[n].base) ||
+            !file_read_int32 (file, &d->cells[n].check))
+        {
+            goto exit_da_cells_created;
+        }
     }
 
     return d;
 
+exit_da_cells_created:
+    free (d->cells);
 exit_da_created:
     free (d);
+exit_file_read:
+    fseek (file, save_pos, SEEK_SET);
     return NULL;
 }
 
+/**
+ * @brief Free double-array data
+ *
+ * @param d : the double-array data
+ *
+ * Free the given double-array data.
+ */
 void
 da_free (DArray *d)
 {
@@ -227,6 +277,18 @@ da_free (DArray *d)
     free (d);
 }
 
+/**
+ * @brief Write double-array data
+ *
+ * @param d     : the double-array data
+ * @param file  : the file to write to
+ *
+ * @return 0 on success, non-zero on failure
+ *
+ * Write double-array data to the given @a file, starting from the current
+ * file pointer. On return, the file pointer is left after the double-array
+ * data block.
+ */
 int
 da_write (const DArray *d, FILE *file)
 {
@@ -244,6 +306,15 @@ da_write (const DArray *d, FILE *file)
 }
 
 
+/**
+ * @brief Get root state
+ *
+ * @param d     : the double-array data
+ *
+ * @return root state of the @a index set, or TRIE_INDEX_ERROR on failure
+ *
+ * Get root state for stepwise walking.
+ */
 TrieIndex
 da_get_root (const DArray *d)
 {
@@ -252,12 +323,32 @@ da_get_root (const DArray *d)
 }
 
 
+/**
+ * @brief Get BASE cell
+ *
+ * @param d : the double-array data
+ * @param s : the double-array state to get data
+ *
+ * @return the BASE cell value for the given state
+ *
+ * Get BASE cell value for the given state.
+ */
 TrieIndex
 da_get_base (const DArray *d, TrieIndex s)
 {
     return (s < d->num_cells) ? d->cells[s].base : TRIE_INDEX_ERROR;
 }
 
+/**
+ * @brief Get CHECK cell
+ *
+ * @param d : the double-array data
+ * @param s : the double-array state to get data
+ *
+ * @return the CHECK cell value for the given state
+ *
+ * Get CHECK cell value for the given state.
+ */
 TrieIndex
 da_get_check (const DArray *d, TrieIndex s)
 {
@@ -265,6 +356,15 @@ da_get_check (const DArray *d, TrieIndex s)
 }
 
 
+/**
+ * @brief Set BASE cell
+ *
+ * @param d   : the double-array data
+ * @param s   : the double-array state to get data
+ * @param val : the value to set
+ *
+ * Set BASE cell for the given state to the given value.
+ */
 void
 da_set_base (DArray *d, TrieIndex s, TrieIndex val)
 {
@@ -273,6 +373,15 @@ da_set_base (DArray *d, TrieIndex s, TrieIndex val)
     }
 }
 
+/**
+ * @brief Set CHECK cell
+ *
+ * @param d   : the double-array data
+ * @param s   : the double-array state to get data
+ * @param val : the value to set
+ *
+ * Set CHECK cell for the given state to the given value.
+ */
 void
 da_set_check (DArray *d, TrieIndex s, TrieIndex val)
 {
@@ -281,6 +390,20 @@ da_set_check (DArray *d, TrieIndex s, TrieIndex val)
     }
 }
 
+/**
+ * @brief Walk in double-array structure
+ *
+ * @param d : the double-array structure
+ * @param s : current state
+ * @param c : the input character
+ *
+ * @return boolean indicating success
+ *
+ * Walk the double-array trie from state @a *s, using input character @a c.
+ * If there exists an edge from @a *s with arc labeled @a c, this function
+ * returns TRUE and @a *s is updated to the new state. Otherwise, it returns
+ * FALSE and @a *s is left unchanged.
+ */
 Bool
 da_walk (const DArray *d, TrieIndex *s, TrieChar c)
 {
@@ -294,6 +417,19 @@ da_walk (const DArray *d, TrieIndex *s, TrieChar c)
     return FALSE;
 }
 
+/**
+ * @brief Insert a branch from trie node
+ *
+ * @param d : the double-array structure
+ * @param s : the state to add branch to
+ * @param c : the character for the branch label
+ *
+ * @return the index of the new node
+ *
+ * Insert a new arc labelled with character @a c from the trie node 
+ * represented by index @a s in double-array structure @a d.
+ * Note that it assumes that no such arc exists before inserting.
+ */
 TrieIndex
 da_insert_branch (DArray *d, TrieIndex s, TrieChar c)
 {
@@ -576,12 +712,33 @@ da_extend_pool     (DArray         *d,
     return TRUE;
 }
 
+/**
+ * @brief Prune the single branch
+ *
+ * @param d : the double-array structure
+ * @param s : the dangling state to prune off
+ *
+ * Prune off a non-separate path up from the final state @a s.
+ * If @a s still has some children states, it does nothing. Otherwise, 
+ * it deletes the node and all its parents which become non-separate.
+ */
 void
 da_prune (DArray *d, TrieIndex s)
 {
     da_prune_upto (d, da_get_root (d), s);
 }
 
+/**
+ * @brief Prune the single branch up to given parent
+ *
+ * @param d : the double-array structure
+ * @param p : the parent up to which to be pruned
+ * @param s : the dangling state to prune off
+ *
+ * Prune off a non-separate path up from the final state @a s to the
+ * given parent @a p. The prunning stop when either the parent @a p
+ * is met, or a first non-separate node is found.
+ */
 void
 da_prune_upto (DArray *d, TrieIndex p, TrieIndex s)
 {
@@ -628,6 +785,20 @@ da_free_cell       (DArray         *d,
     da_set_base (d, i, -cell);
 }
 
+/**
+ * @brief Enumerate entries stored in double-array structure
+ *
+ * @param d          : the double-array structure
+ * @param enum_func  : the callback function to be called on each separate node
+ * @param user_data  : user-supplied data to send as an argument to @a enum_func
+ *
+ * @return boolean value indicating whether all the keys are visited
+ *
+ * Enumerate all keys stored in double-array structure. For each entry, the 
+ * user-supplied @a enum_func callback function is called, with the entry key,
+ * the separate node, and user-supplied data. Returning FALSE from such
+ * callback will stop enumeration and return FALSE.
+ */
 Bool
 da_enumerate (const DArray *d, DAEnumFunc enum_func, void *user_data)
 {
